@@ -131,15 +131,14 @@ end
 
 if iscell (args)
   x = args{1};
-
-##if is_list (args),		# List of arguments 
-##  x = nth (args, narg);
+elseif islist (args),		# List of arguments 
+  args = {args{:}};
+  x = args{1};
 else				# Single argument
   x = args;
-  #### args = list (args); 
   args = {args};
-end
-## args
+endif
+
 
 if narg > length (args)		# Check
   error ("nelder_mead_min : narg==%i, length (args)==%i\n",
