@@ -26,52 +26,11 @@
 #include <oct.h>
 #include <octave/parse.h>
 #include <octave/Cell.h>
-#include <octave/lo-ieee.h>
 #include <float.h>
 #include "error.h"
-
-// the compares two octave_values (thanks to jwe)
-bool isequal (const octave_value& a, const octave_value& b,
-           bool nans_compare_equal = false)
-{
-    octave_value_list args;
-
-    args(2) = b;
-    args(1) = a;
-    args(0) = nans_compare_equal;
-
-    octave_value_list tmp = feval ("__isequal__", args, 1);
-
-    // We expect __isequal__ to always return a single logical scalar.
-    return tmp(0).bool_value ();
-}
-
-// this is cache on feval to prevent the extra evaluation of the
-// objective function when subsequent calls with exactly the same arguments would take place
-
-octave_value_list _feval( const std::string f, const octave_value_list f_args )
-{
-    static octave_value_list f_args_prev;
-    static octave_value_list ret;
-
-    // check if ( f_args_prev == f_args )
-    bool same = true;
-    for( int i=0; i < f_args.length(); i++)
-    {
-        if (!isequal( f_args(i),f_args_prev(i)))
-        {
-            same = false;
-            break;
-        }
-    }
-    if (same)
-        return ret;
-    
-    f_args_prev = f_args;
-    ret = feval( f, f_args );
-
-    return ret;
-}
+#ifdef __MINGW32__
+#define isnan _isnan
+#endif
 
 int __bfgsmin_obj(double &obj, const std::string f, const octave_value_list f_args, const ColumnVector theta, const int minarg)
 {
@@ -79,7 +38,7 @@ int __bfgsmin_obj(double &obj, const std::string f, const octave_value_list f_ar
 	int success = 1;
 	f_args_new = f_args;
 	f_args_new(minarg - 1) = theta;
-	f_return = _feval(f, f_args_new);
+	f_return = feval(f, f_args_new);
 	obj = f_return(0).double_value();
 	// bullet-proof the objective function
 	if (error_state) {
@@ -132,12 +91,12 @@ int __bfgsmin_gradient(ColumnVector &derivative, const std::string f, octave_val
 	Matrix check_gradient(k,1);
 	if (have_analytic_gradient) {
 		f_args(minarg - 1) = theta;
-		f_return = _feval(f, f_args);
+		f_return = feval(f, f_args);
 		g = f_return(1).column_vector_value();
 	}
 	else if (try_analytic_gradient) {
 		f_args(minarg - 1) = theta;
-		f_return = _feval(f, f_args);
+		f_return = feval(f, f_args);
 		if (f_return.length() > 1) {
 			if (f_return(1).is_real_matrix()) {
         			if ((f_return(1).rows() == k) & (f_return(1).columns() == 1)) {
@@ -282,7 +241,7 @@ int __newtonstep(double &step, double &obj, const std::string f, const octave_va
 		found_improvement = __bisectionstep(step, obj, f, f_args, x, dx, minarg, verbose);
 	}
 	else found_improvement = 1;
-	if (lo_ieee_isnan(obj)) {
+	if (isnan(obj)) {
 		obj = obj_0;
 		if (verbose) warning("__stepsize: objective function crash in Newton step, falling back to bisection");
 		found_improvement = __bisectionstep(step, obj, f, f_args, x, dx, minarg, verbose);
@@ -321,9 +280,9 @@ Users should not use this directly. Use bfgsmin.m instead") {
 	warnings = 0;
 	if (verbosity == 3) warnings = 1;
 
-	// copy cell contents over to octave_value_list to use _feval()
+	// copy cell contents over to octave_value_list to use feval()
 	k = f_args_cell.length();
-	if (k) f_args(k-1); // resize only once
+	f_args(k); // resize only once
 	for (i = 0; i<k; i++) f_args(i) = f_args_cell(i);
 
 	// get the minimization argument
@@ -349,11 +308,14 @@ Users should not use this directly. Use bfgsmin.m instead") {
 
 	// Initial gradient (try analytic, and use it if it's close enough to numeric)
 	__bfgsmin_gradient(g, f, f_args, theta, minarg, 1, have_gradient);	// try analytic
+	if (warnings) printf("function claims to provide analytic gradient\n");
 	if (have_gradient) {					// check equality if analytic available
 		have_gradient = 0;				// force numeric
 		__bfgsmin_gradient(g_new, f, f_args, theta, minarg, 0, have_gradient);
 		p = g - g_new;
 		have_gradient = sqrt(p.transpose() * p) < gradient_tol;
+		if (have_gradient && warnings) printf("function claims to provide analytic gradient, and it agrees with numeric - using analytic\n");
+		if (!have_gradient && warnings) printf("function claims to provide analytic gradient, but it does not agree with numeric - using numeric\n");
 	}
 
 	last_obj_value = obj_in; // initialize, is updated after each iteration
@@ -412,8 +374,8 @@ Users should not use this directly. Use bfgsmin.m instead") {
 				if (memory > 0) printf("Using LBFGS, memory is last %d iterations\n",memory);
 			}
 			printf("\nfunction value: %g  stepsize: %g  \n\n", obj_value, stepsize);
-			if (have_gradient) printf("          param    gradient (n)          change\n");
-			else printf("          param    gradient (n)          change\n");
+			if (have_gradient) printf("used analytic gradient\n");
+			else printf("used numeric gradient\n");
 			for (j = 0; j<k; j++) printf("%15.5f %15.5f %15.5f\n",theta(j), g(j), p(j));
 		}
 		// Are we done?
@@ -491,8 +453,9 @@ Users should not use this directly. Use bfgsmin.m instead") {
 		if ((convergence == 1) & !(criterion == 1)) printf("WEAK CONVERGENCE\n");
 		if (convergence == 2)                       printf("NO CONVERGENCE: algorithm failed\n");
 		printf("Function conv %d  Param conv %d  Gradient conv %d\n\n", conv_fun, conv_param, conv_grad);
-		if (have_gradient) printf("          param    gradient (n)          change\n");
-		else printf("          param    gradient (n)          change\n");
+		if (have_gradient) printf("used analytic gradient\n");
+		else printf("used numeric gradient\n");
+		printf("          param    gradient (n)          change\n");
 		for (j = 0; j<k; j++) printf("%15.5f %15.5f %15.5f\n",theta(j), g(j), p(j));
 	}
 	f_return(3) = iter;
